{see rmAGraph.txt and rmAGraph.hlp for more details}

unit rmAGraph;

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs,  ExtCtrls, clipbrd;

type
  TGridType = (gtNone, gtVertical, gtHorizontal, gtBoth);
  TBorderType = (btNone, btRaised, btLowered, btSingle);
  TGridSubDivType = (sdNone, sdVertical, sdHorizontal, sdBoth);
  TValuePosition = (vpPixels, vpFullWidth);
  TTracesType = (Trace1, Trace2, Trace3, Trace4, Trace5, Trace6, Trace7, Trace8);
  TTextType = (stUseHMinMax, stUseVMinMax, stLeft, stRight, stBottom, stTop, stHorzOnlyExtremity, stVertOnlyExtremity);
  TScaleText = set of TTextType;
  TTraces = set of TTracesType;

  TrmAGraph = class(TGraphicControl)
  private
    FBmpBuffer: TBitmap;
    FGridType: TGridType;
    FGridSubDiv: TGridSubDivType;
    FGridSubDivLen: word;
    FCTL3D: boolean;
    FShowOver: boolean;
    FValuePosition: TValuePosition;
    FBGColor: TColor;
    FBorder: TBorderType;
    FGraphColor: TColor;
    FGridStyle: TPenStyle;
    FGridColor: TColor;
    FTrace1Color, FTrace2Color, FTrace3Color, FTrace4Color,
    FTrace5Color, FTrace6Color, FTrace7Color, FTrace8Color: TColor;
    FLeftMargin, FRightMargin, FTopMargin, FBottomMargin: integer;
    FShowVertLimit: boolean;
    FVertLimitColor: TColor;
    FVertMinLimit, FVertMaxLimit: single;
    FHorzMinVal, FVertMinVal: single;
    FHorzMaxVal, FVertMaxVal: single;
    FHorzMinText, FVertMinText: string;
    FHorzMaxText, FVertMaxText: string;
    FHorzUnit, FVertUnit: string;
    FHorzDecNum, FVertDecNum: integer;
    FHorzDiv, FVertDiv: integer;
    FScaleText: TScaleText;
    FTraces: TTraces;
    FTextColor: TColor;
    FTextSize: integer;
    FRangeUse, FRangeMarker, FRangeAutoMove: boolean;
    FRangeStart, FRangeStop: integer;
    FMarkerColor: TColor;
    FMaxItems: integer;
    procedure SetGridType(value: TGridType);
    procedure SetGridSubDiv(value: TGridSubDivType);
    procedure SetGridSubDivLen(value: word);
    procedure SetCTL3D(value: boolean);
    procedure SetShowOver(value: boolean);
    procedure SetValuePosition(value: TValuePosition);
    procedure SetLstValues(value: TStringList);
    procedure SetBGColor(value: TColor);
    procedure SetBorder(value: TBorderType);
    procedure SetGraphColor(value: TColor);
    procedure SetGridStyle(value: TPenStyle);
    procedure SetGridColor(value: TColor);
    procedure SetTrace1Color(value: TColor);
    procedure SetTrace2Color(value: TColor);
    procedure SetTrace3Color(value: TColor);
    procedure SetTrace4Color(value: TColor);
    procedure SetTrace5Color(value: TColor);
    procedure SetTrace6Color(value: TColor);
    procedure SetTrace7Color(value: TColor);
    procedure SetTrace8Color(value: TColor);
    procedure SetLeftMargin(value: integer);
    procedure SetRightMargin(value: integer);
    procedure SetTopMargin(value: integer);
    procedure SetBottomMargin(value: integer);
    procedure SetHorzMinVal(value: single);
    procedure SetHorzMaxVal(value: single);
    procedure SetHorzMinText(value: string);
    procedure SetHorzMaxText(value: string);
    procedure SetHorzUnit(value: string);
    procedure SetHorzDecNum(value: integer);
    procedure SetHorzDiv(value: integer);
    procedure SetShowVertLimit(value: boolean);
    procedure SetVertLimitColor(value: TColor);
    procedure SetVertMinLimit(value: single);
    procedure SetVertMaxLimit(value: single);
    procedure SetVertMinVal(value: single);
    procedure SetVertMaxVal(value: single);
    procedure SetVertMinText(value: string);
    procedure SetVertMaxText(value: string);
    procedure SetVertUnit(value: string);
    procedure SetVertDecNum(value: integer);
    procedure SetVertDiv(value: integer);
    procedure SetScaleText(value: TScaleText);
    procedure SetTraces(value: TTraces);
    procedure SetTextColor(value: TColor);
    procedure SetTextSize(value: integer);
    procedure SetRangeUse(value: boolean);
    procedure SetRangeAutoMove(value: boolean);
    procedure SetRangeMarker(value: boolean);
    procedure SetRangeStart(value: integer);
    procedure SetRangeStop(value: integer);
    procedure SetMarkerColor(value: TColor);
    procedure SetMaxItems(value: integer);
  protected
    OriginPosMarker: integer;
    NewPosMarker: integer;
    procedure Paint; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure UpdateBmpBuffer(colored: boolean);
  public
    constructor Create(anOwner:TComponent); override;
    destructor Destroy; override;
    procedure Free;
  published
    LstValues: TStringList;
    property GridType: TGridType read FGridType write SetGridType;
    property GridSubDiv: TGridSubDivType read FGridSubDiv write SetGridSubDiv;
    property GridSubDivLen: word read FGridSubDivLen write SetGridSubDivLen;
    property ValuePosition: TValuePosition read FValuePosition write SetValuePosition;
    // property Values: TStringList read FValues write SetValues;
    property BGColor: TColor read FBGColor write SetBGColor;
    property CTL3D: boolean read FCTL3D write SetCTL3D;
    property ShowOver: boolean read FShowOver write SetShowOver;
    property Border: TBorderType read FBorder write SetBorder;
    property GraphColor: TColor read FGraphColor write SetGraphColor;
    property GridStyle: TPenStyle read FGridStyle write SetGridStyle;
    property GridColor: TColor read FGridColor write SetGridColor;
    property Trace1Color: TColor read FTrace1Color write SetTrace1Color;
    property Trace2Color: TColor read FTrace2Color write SetTrace2Color;
    property Trace3Color: TColor read FTrace3Color write SetTrace3Color;
    property Trace4Color: TColor read FTrace4Color write SetTrace4Color;
    property Trace5Color: TColor read FTrace5Color write SetTrace5Color;
    property Trace6Color: TColor read FTrace6Color write SetTrace6Color;
    property Trace7Color: TColor read FTrace7Color write SetTrace7Color;
    property Trace8Color: TColor read FTrace8Color write SetTrace8Color;
    property LeftMargin: integer read FLeftMargin write SetLeftMargin;
    property RightMargin: integer read FRightMargin write SetRightMargin;
    property TopMargin: integer read FTopMargin write SetTopMargin;
    property BottomMargin: integer read FBottomMargin write SetBottomMargin;
    property HorzMinVal: single read FHorzMinVal write SetHorzMinVal;
    property HorzMaxVal: single read FHorzMaxVal write SetHorzMaxVal;
    property HorzMinText: string read FHorzMinText write SetHorzMinText;
    property HorzMaxText: string read FHorzMaxText write SetHorzMaxText;
    property HorzUnit: string read FHorzUnit write SetHorzUnit;
    property HorzDiv: integer read FHorzDiv write SetHorzDiv;
    property HorzDecNum: integer read FHorzDecNum write SetHorzDecNum;
    property ShowVertLimit: boolean read FShowVertLimit write SetShowVertLimit;
    property VertLimitColor: TColor read FVertLimitColor write SetVertLimitColor;
    property VertMinLimit: single read FVertMinLimit write SetVertMinLimit;
    property VertMaxLimit: single read FVertMaxLimit write SetVertMaxLimit;
    property VertMinVal: single read FVertMinVal write SetVertMinVal;
    property VertMaxVal: single read FVertMaxVal write SetVertMaxVal;
    property VertMinText: string read FVertMinText write SetVertMinText;
    property VertMaxText: string read FVertMaxText write SetVertMaxText;
    property VertUnit: string read FVertUnit write SetVertUnit;
    property VertDecNum: integer read FVertDecNum write SetVertDecNum;
    property VertDiv: integer read FVertDiv write SetVertDiv;
    property ScaleText: TScaleText read FScaleText write SetScaleText;
    property Traces: TTraces read FTraces write SetTraces;
    property TextColor: TColor read FTextColor write SetTextColor;
    property TextSize: integer read FTextSize write SetTextSize;
    property RangeUse: boolean read FRangeUse write SetRangeUse;
    property RangeAutoMove: boolean read FRangeAutoMove write SetRangeAutoMove;
    property RangeMarker: boolean read FRangeMarker write SetRangeMarker;
    property RangeStart: integer read FRangeStart write SetRangeStart;
    property RangeStop: integer read FRangeStop write SetRangeStop;
    property MarkerColor: TColor read FMarkerColor write SetMarkerColor;
    property MaxItems: integer read FMaxItems write SetMaxItems;

    procedure LoadDataFromFile(sFile: TFileName);
    procedure SaveDataToFile(sFile: TFileName);
    procedure SaveGraphToFile(colored: boolean; sFile: TFileName);
    procedure CopyGraphToClipboard(colored: boolean);
    procedure DrawPositionMarker(x: integer);
    procedure ClearMarker;
    procedure Clear;
    function AddValues(s: string; refresh: boolean): integer;
    function DeleteValues(i: integer; refresh: boolean): integer;
    function GetValues(x, y: integer; var ValArray: array of real): byte;

    function GraphBottom: integer;
    function GraphLeft: integer;
    function GraphRight: integer;
    function GraphTop: integer;
    function GraphHeight: integer;
    function GraphWidth: integer;

    { Standard properties }
    property Align;
    property Height;
    property Width;
    property PopupMenu;

    { Standard events }
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('OptoElec', [TrmAGraph]);
end;

function r2s(dec: integer; r: real): string;
var
  s: string;
begin
  str(r:0:dec, s);
  result := s;
end;

function s2r(s: string): real;
var
  r: real;
  u: integer;
begin
  val(s, r, u);
  result := r;
end;

constructor TrmAGraph.Create(anOwner:TComponent);
begin
  Inherited Create(anOwner);
  FBmpBuffer := TBitmap.Create;
  OriginPosMarker := -1;
  NewPosMarker := -1;
  LstValues := TStringList.Create;
  FBorder := btRaised;
  FCTL3D := true;
  FShowOver := true;
  FBGColor := clSilver;
  FGraphColor := clBlack;
  FGridStyle := psDot;
  FGridColor := clGreen;
  FTrace1Color := clLime;
  FTrace2Color := clRed;
  FTrace3Color := clAqua;
  FTrace4Color := clYellow;
  FTrace5Color := clFuchsia;
  FTrace6Color := clBlue;
  FTrace7Color := clGreen;
  FTrace8Color := clSilver;
  Height := 305;
  Width := 433;
  FGridType := gtBoth;
  FGridSubDiv := sdNone;
  FGridSubDivLen := 3;
  FValuePosition := vpPixels;
  FLeftMargin := 45;
  FRightMargin := 4;
  FTopMargin := 4;
  FBottomMargin := 20;
  FHorzMinVal := 0;
  FHorzMaxVal := 100;
  FHorzMinText := 'HMin';
  FHorzMaxText := 'HMax';
  FHorzUnit := '';
  FHorzDecNum := 0;
  FHorzDiv := 10;
  FShowVertLimit := false;
  FVertLimitColor := clRed;
  FVertMinLimit := 30;
  FVertMaxLimit := 80;
  FVertMinVal := 0;
  FVertMaxVal := 100;
  FVertMinText := 'VMin';
  FVertMaxText := 'VMax';
  FVertUnit := '';
  FVertDecNum := 1;
  FRangeUse := false;
  FRangeAutoMove := false;
  FRangeMarker := false;
  FRangeStart := 0;
  FRangeStop := 100;
  FMarkerColor := clRed;
  FVertDiv := 10;
  FScaleText := [stUseVMinMax, stLeft, stBottom];
  FTraces := [Trace1, Trace2, Trace3, Trace4];
  FTextColor := clBlue;
  FTextSize := 8;
  FMaxItems := 0;
end;

destructor TrmAGraph.Destroy;
begin
  If FBmpBuffer <> NIL then FBmpBuffer.free;
  if LstValues <> nil then LstValues.Free;
  Inherited Destroy;
end;

procedure TrmAGraph.Free;
begin
  FBmpBuffer.Free;
  LstValues.free;
  Inherited Free;
end;

procedure TrmAGraph.SetBounds(aLeft, aTop, aWidth, aHeight: Integer);
begin
  Inherited ;
  If FBmpBuffer <> NIL then begin
    FBmpBuffer.Destroy;
    FBmpBuffer := TBitmap.Create;
    FBmpBuffer.Height := aHeight;
    FBmpBuffer.Width := aWidth;
    OriginPosMarker := -1;
    NewPosMarker := -1;
  end;
end;

procedure TrmAGraph.UpdateBmpBuffer(colored: boolean);

  procedure DrawBackGround;
  begin
    With FBmpBuffer.Canvas do begin
      if colored then begin
        if (FBorder <> btSingle) then Pen.Color := FBGColor
        else Pen.Color := clBlack;
        Brush.Color := FBGColor;
      end else begin
        if (FBorder <> btNone) then Pen.Color := clBlack
        else pen.color := clWhite;
        Brush.Color := clWhite;
      end;
      Rectangle(0, 0, width, height);

      // 3D border only if colored
      if colored then begin
        if (FBorder = btRaised) or (FBorder = btLowered) then begin
          if FBorder = btRaised then Pen.Color := clWhite else pen.color := clGray;
          moveTo(0, height - 1);
          lineTo(0, 0);
          lineTo(width - 1, 0);
          if FBorder = btRaised then Pen.Color := clGray else pen.color := clWhite;
          lineTo(width - 1, height - 1);
          lineTo(0, height - 1);
        end;
      end;
    end;
  end;

  procedure DrawGraph;
  var
    GraphW, GraphH: integer;
  begin
    graphW := GraphWidth;
    graphH := GraphHeight;
    With FBmpBuffer.Canvas do begin
      if colored then begin
        Pen.Color := FGraphColor;
        Brush.Color := FGraphColor;
      end else begin
        Pen.Color := clBlack;
        Brush.Color := clWhite;
      end;
      Rectangle(FLeftMargin, FTopMargin, width - FRightMargin, Height - bottomMargin);

      // 3D border on draw zone only if colored
      if colored and FCTL3D then begin
        Pen.Color := clGray;
        moveTo(leftMargin - 1, TopMargin + graphH);
        lineTo(leftMargin - 1, TopMargin - 1);
        lineTo(leftMargin + graphW, TopMargin - 1);
        Pen.Color := clWhite;
        lineTo(leftMargin + graphW, TopMargin  + graphH);
        lineTo(leftMargin - 1, TopMargin + graphH);
      end;
    end;
  end;

  procedure DrawGrid;
  var
    i: integer;
    HPos, VPos: integer;
    GraphW, GraphH: integer;
  begin
    graphW := GraphWidth;
    graphH := GraphHeight;
    With FBmpBuffer.Canvas do begin
      pen.style := FGridStyle;
      if colored then Pen.Color := FGridColor else Pen.Color := clBlack;
      if (FGridType = gtHorizontal) or (FGridType = gtBoth) then begin
        for i := 1 to FHorzDiv - 1 do begin
          HPos := FLeftMargin + trunc((i * GraphW) / FHorzDiv);
          MoveTo(HPos, FTopMargin);
          LineTo(HPos, Height - FBottomMargin);
        end;
        // Horizontal Sub-divisions
        if (FGridSubDiv = sdHorizontal) or (FGridSubDiv = sdBoth) then begin
          for i := 1 to (FHorzDiv * 10) - 1 do begin
            HPos := FLeftMargin + trunc((i * GraphW) / (FHorzDiv * 10));
            MoveTo(HPos, Height - FBottomMargin - FGridSubDivLen);
            LineTo(HPos, Height - FBottomMargin);
          end;
        end;
      end;
      if (FGridType = gtVertical) or (FGridType = gtBoth) then begin
        for i := 1 to FVertDiv - 1 do begin
          VPos := FTopMargin + trunc((i * GraphH) / FVertDiv);
          MoveTo(FLeftMargin, VPos);
          LineTo(width - FRightMargin, VPos);
        end;
        // Vertical Sub-divisions
        if (FGridSubDiv = sdVertical) or (FGridSubDiv = sdBoth) then begin
          for i := 1 to (FVertDiv * 10) - 1 do begin
            VPos := FTopMargin + trunc((i * GraphH) / (FVertDiv * 10));
            MoveTo(FLeftMargin, VPos);
            LineTo(FLeftMargin + FGridSubDivLen, VPos);
          end;
        end;
      end;
      pen.style := psSolid;
    end;
  end;

  procedure DrawScaleText;
  var
    i: integer;
    tmpVal: real;
    s: string;
    HPos, VPos: integer;
    GraphH, GraphW: integer;
  begin
    graphW := GraphWidth;
    graphH := GraphHeight;
    With FBmpBuffer.Canvas do begin
      // left, right, top and bottom scale text
      if (stLeft in FScaleText) or (stRight in FScaleText) then begin
        font.Name := 'Arial';
        font.Size := FTextSize;
        if colored then begin
          brush.Color := FBGColor;
          font.Color := FTextColor;
        end else begin
          brush.Color := clWhite;
          font.Color := clBlack;
        end;
        if (stUseVMinMax in FScaleText) then begin
          for i := 0 to FVertDiv do begin
            if (i <> 0) and (i <> FVertDiv) and (stVertOnlyExtremity in FScaleText) then
              continue;
            tmpVal := FVertMaxVal - ((FVertMaxVal - FVertMinVal) / FVertDiv) * i;
            s := r2s(FVertDecNum, tmpVal) + FVertUnit;
            if i = 0 then
              VPos := FTopMargin
            else if i = FVertDiv then
              VPos := FTopMargin + GraphH - textHeight(s) + 2
            else
              VPos := FTopMargin + trunc((i * GraphH) / FVertDiv) - (textHeight(s) div 2);
            if (stLeft in FScaleText) then
              textOut(FLeftMargin - 4 - textWidth(s), VPos, s);
            if (stRight in FScaleText) then
              textOut(FLeftMargin + graphW + 4, VPos, s);
          end;
        end else begin
          s := FVertMaxText;
          VPos := FTopMargin;
          if (stLeft in FScaleText) then
            textOut(FLeftMargin - 4 - textWidth(s), VPos, s);
          if (stRight in FScaleText) then
            textOut(FLeftMargin + graphW + 4, VPos, s);
          s := FVertMinText;
          VPos := FTopMargin + GraphH - textHeight(s) + 2;
          if (stLeft in FScaleText) then
            textOut(FLeftMargin - 4 - textWidth(s), VPos, s);
          if (stRight in FScaleText) then
            textOut(FLeftMargin + graphW + 4, VPos, s);
        end;
      end;
      if (stTop in FScaleText) or (stBottom in FScaleText) then begin
        font.Size := FTextSize;
        font.Name := 'Arial';
        if colored then begin
          brush.Color := FBGColor;
          font.Color := FTextColor;
        end else begin
          brush.Color := clWhite;
          font.Color := clBlack;
        end;
        if (stUseHMinMax in FScaleText) then begin
          for i := 0 to FHorzDiv do begin
            if (i <> 0) and (i <> FHorzDiv) and (stHorzOnlyExtremity in FScaleText) then
              continue;
            if (i = 0) or (i = FHorzDiv) then begin
              tmpVal := FHorzMinVal + ((FHorzMaxVal - FHorzMinVal) / FHorzDiv) * i;
              s := r2s(FHorzDecNum, tmpVal);
            end else begin
              tmpVal := FHorzMinVal + ((FHorzMaxVal - FHorzMinVal) / FHorzDiv) * i;
              s := r2s(FHorzDecNum, tmpVal) + FHorzUnit;
            end;
            if i = 0 then
              HPos := FLeftMargin - 2
            else if i = FHorzDiv then
              HPos := FLeftMargin + GraphW - textWidth(s)
            else
              HPos := FLeftMargin + trunc((i * GraphW) / FHorzDiv) - (textWidth(s) div 2);
            if (stTop in FScaleText) then
              textOut(HPos, FTopMargin - 4 - textHeight(s), s);
            if (stBottom in FScaleText) then
              textOut(HPos, FTopMargin + graphH + 4, s);
          end;
        end else begin
          s := FHorzMinText;
          HPos := FLeftMargin - 2;
          if (stTop in FScaleText) then
            textOut(HPos, FTopMargin - 4 - textHeight(s), s);
          if (stBottom in FScaleText) then
            textOut(HPos, FTopMargin + graphH + 4, s);
          s := FHorzMaxText;
          HPos := FLeftMargin + GraphW - textWidth(s);
          if (stTop in FScaleText) then
            textOut(HPos, FTopMargin - 4 - textHeight(s), s);
          if (stBottom in FScaleText) then
            textOut(HPos, FTopMargin + graphH + 4, s);
        end;
      end;
    end;
  end;

  procedure DrawLimitLines;
  var
    i: integer;
    GraphH, GraphW: integer;
    VPos: integer;
  begin
    if not FShowVertLimit then exit;
    graphW := GraphWidth;
    graphH := GraphHeight;
    With FBmpBuffer.Canvas do begin
      pen.Color := FVertLimitColor;
      pen.style := psDot;

      // Min Limit vertical position
      VPos := FTopMargin + trunc(((FVertMaxVal - FVertMinLimit) * GraphH) / (FVertMaxVal - FVertMinVal));
      if (VPos < (FTopMargin + graphH - 1)) and (VPos > FTopMargin) then begin
        MoveTo(FLeftMargin, VPos);
        lineTo(width - FRightMargin, VPos);
      end;

      // Max Limit vertical position
      VPos := FTopMargin + trunc(((FVertMaxVal - FVertMaxLimit) * GraphH) / (FVertMaxVal - FVertMinVal));
      if (VPos < (FTopMargin + graphH - 1)) and (VPos > FTopMargin) then begin
        MoveTo(FLeftMargin, VPos);
        lineTo(width - FRightMargin, VPos);
      end;

      pen.style := psSolid;
    end;
  end;

  procedure DrawConceptionTraces;
  var
    s: string;
    i: integer;
    tmpColor: TColor;
    GraphH, GraphW: integer;
  begin
    graphW := GraphWidth;
    graphH := GraphHeight;
    With FBmpBuffer.Canvas do begin
      // traces, only in conception (to see effects of colors choice)
      if csDesigning in ComponentState then begin
        brush.Color := GraphColor;
        font.size := FTextSize;
        for i := 1 to 8 do begin
          case i of
            1 : if not (trace1 in FTraces) then continue;
            2 : if not (trace2 in FTraces) then continue;
            3 : if not (trace3 in FTraces) then continue;
            4 : if not (trace4 in FTraces) then continue;
            5 : if not (trace5 in FTraces) then continue;
            6 : if not (trace6 in FTraces) then continue;
            7 : if not (trace7 in FTraces) then continue;
            8 : if not (trace8 in FTraces) then continue;
          end;
          case i of
            1 : tmpColor := Trace1Color;
            2 : tmpColor := Trace2Color;
            3 : tmpColor := Trace3Color;
            4 : tmpColor := Trace4Color;
            5 : tmpColor := Trace5Color;
            6 : tmpColor := Trace6Color;
            7 : tmpColor := Trace7Color;
            8 : tmpColor := Trace8Color;
          end;
          pen.Color := tmpColor;
          font.Color := tmpColor;
          moveTo(FLeftMargin, FTopMargin + (graphH div 10) * i);
          lineTo(FLeftMargin + GraphW, FTopMargin + (graphH div 10) * i);
          s := 'Trace ' + intToStr(i) + ' - Conception';
          textOut(FLeftMargin + 5, FTopMargin + (graphH div 10) * i - textHeight(s) - 2, s);
        end;
      end;
    end;
  end;

  procedure DrawExecutionTraces;
  var
    s: string;
    i, j: integer;
    start, stop: integer;
    sep: integer;
    tmpColor: TColor;
    HPos, VPos: integer;
    ValuesTmp: TStringList;
    Empty: boolean;
    tmpVal: real;
    UpOver, DownOver: boolean;
    graphW, graphH: integer;
  begin
    DownOver := false;
    UpOver := false;
    graphW := GraphWidth;
    graphH := GraphHeight;
    try
      ValuesTmp := TStringList.Create;

    With FBmpBuffer.Canvas do begin
      // use all values or only values in range ?
      if FRangeUse then begin
        Start := FRangeStart;
        if FRangeStop > (LstValues.Count - 1) then
          stop := LstValues.Count - 1
        else
          stop := FRangeStop;
      end else begin
        Start := 0;
        Stop := LstValues.Count - 1;
      end;

      // decode values in StringList LstValues...
      if LstValues.Count > 0 then begin
        ValuesTmp.Assign(LstValues);
        empty := false;
        for i := 1 to 8 do begin
          if empty then break;
          for j := Start to Stop do begin
            s := ValuesTmp[j];
            case i of
              1 : if colored then Pen.Color := FTrace1Color else pen.color := clBlack;
              2 : if colored then Pen.Color := FTrace2Color else pen.color := clBlack;
              3 : if colored then Pen.Color := FTrace3Color else pen.color := clBlack;
              4 : if colored then Pen.Color := FTrace4Color else pen.color := clBlack;
              5 : if colored then Pen.Color := FTrace5Color else pen.color := clBlack;
              6 : if colored then Pen.Color := FTrace6Color else pen.color := clBlack;
              7 : if colored then Pen.Color := FTrace7Color else pen.color := clBlack;
              8 : if colored then Pen.Color := FTrace8Color else pen.color := clBlack;
            end;
            sep := pos(';', s);
            if (sep = 0) and (s = '') then begin
              Empty := true;
              break;
            end;
            if sep > 0 then begin
              tmpVal := s2r(copy(s, 1, sep - 1));
              delete(s, 1, sep);
            end else begin
              tmpVal := s2r(s);
              s := '';
            end;
            ValuesTmp[j] := s;
            // if (val > FVertMaxVal) or (val < FVertMinVal) then continue;

            if (Trace1 in FTraces) and (i = 1)
            or (Trace2 in FTraces) and (i = 2)
            or (Trace3 in FTraces) and (i = 3)
            or (Trace4 in FTraces) and (i = 4)
            or (Trace5 in FTraces) and (i = 5)
            or (Trace6 in FTraces) and (i = 6)
            or (Trace7 in FTraces) and (i = 7)
            or (Trace8 in FTraces) and (i = 8)
            then begin

              // calculate horizontal position
              if FValuePosition = vpPixels then begin
                if (j - start) > GraphW then break;
                HPos := FLeftMargin + (j - start);
              end else begin
                if (j - start) < 2 then
                  HPos := FLeftMargin
                else
                  HPos := FLeftMargin + trunc((GraphW * (j - start)) / (stop - start));
              end;

              // calculate vertical position
              VPos := FTopMargin + trunc(((FVertMaxVal - tmpVal) * GraphH) / (FVertMaxVal - FVertMinVal));
              if VPos > (FTopMargin + graphH - 1) then begin
                VPos := FTopMargin + graphH - 1;
                DownOver := true;
              end;
              if VPos < FTopMargin then begin
                VPos := FTopMargin;
                UpOver := true;
              end;

              // draw data line
              if (j - start) = 0 then MoveTo(HPos, VPos);
              lineTo(HPos, VPos);
            end;
          end;
        end;

        // draw range marker ?
        if FRangeMarker then begin
          Pen.Color := FMarkerColor;
          MoveTo(FLeftMargin + FRangeStart, FTopMargin);
          lineTo(FLeftMargin + FRangeStart, Height - bottomMargin);
          MoveTo(FLeftMargin + FRangeStop, FTopMargin);
          lineTo(FLeftMargin + FRangeStop, Height - bottomMargin);
        end;

        // Over ?
        if FShowOver and (UpOver or (csDesigning in componentState)) then begin
          if colored then begin
            pen.color := clRed;
            brush.color := clRed;
          end else begin
            pen.color := clBlack;
            brush.color := clBlack;
          end;
          Polygon([Point(FLeftMargin + graphW - 8, FTopMargin + 4),
                   Point(FLeftMargin + graphW - 4, FTopMargin + 8),
                   Point(FLeftMargin + graphW - 12, FTopMargin + 8)]);
          end;
        if FShowOver and (DownOver or (csDesigning in componentState)) then begin
          if colored then begin
            pen.color := clRed;
            brush.color := clRed;
          end else begin
            pen.color := clBlack;
            brush.color := clBlack;
          end;
          Polygon([Point(FLeftMargin + graphW - 8, FTopMargin + 14),
                   Point(FLeftMargin + graphW - 4, FTopMargin + 10),
                   Point(FLeftMargin + graphW - 12, FTopMargin + 10)]);
        end;
      end;
    end;
    finally
      valuesTmp.Free;
    end;
  end;


begin
  DrawBackGround;
  DrawGraph;
  DrawGrid;
  DrawScaleText;
  DrawLimitLines;
  DrawConceptionTraces;
  DrawExecutionTraces;
end;

procedure TrmAGraph.Paint;
begin
  try
    UpdateBmpBuffer(true);
  except
  end;
  try
    Canvas.Draw(0, 0, FBmpBuffer);
    DrawPositionMarker(originPosMarker);
  except
  end;
end;

procedure TrmAGraph.LoadDataFromFile(sFile: TFileName);
begin
  try
    LstValues.LoadFromFile(sFile);
    Paint;
  except
    showMessage('Can''t load data from file !');;
    Paint;
  end;
end;

procedure TrmAGraph.SaveDataToFile(sFile: TFileName);
begin
  if LstValues.count > 0 then begin
    try
      LstValues.SaveToFile(sFile);
      Paint;
    except
      showMessage('Can''t save data to file !');;
      Paint;
    end;
  end else
    showMessage('No data to save !');;
end;

procedure TrmAGraph.SaveGraphToFile(colored: boolean; sFile: TFileName);
begin
  try
    UpdateBmpBuffer(colored);
    if FBmpBuffer <> nil then begin
      try
        FBmpBuffer.SaveToFile(sFile);
      except
      end;
    end;
  except
    showMessage('Can''t save graph to file !');
  end;
  paint;
end;

procedure TrmAGraph.CopyGraphToClipboard(colored: boolean);
begin
  try
    UpdateBmpBuffer(colored);
    if FBmpBuffer <> nil then begin
      try
        Clipboard.Assign(FBmpBuffer);
      except
      end;
    end;
  except
    showMessage('Can''t copy graph to clipboard !');
  end;
  paint;
end;

procedure TrmAGraph.SetValuePosition(value: TValuePosition);
begin
  if value <> FValuePosition then
  begin
    FValuePosition := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetLstValues(value: TStringList);
begin
  if value <> LstValues then
  begin
    LstValues := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetBGColor(value: TColor);
begin
  if value <> FBGColor then
  begin
    FBGColor := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetBorder(value: TBorderType);
begin
  if value <> FBorder then
  begin
    FBorder := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetCTL3D(value: boolean);
begin
  if value <> FCTL3D then
  begin
    FCTL3D := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetShowOver(value: boolean);
begin
  if value <> FShowOver then
  begin
    FShowOver := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetGraphColor(value: TColor);
begin
  if value <> FGraphColor then
  begin
    FGraphColor := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetGridStyle(value: TPenStyle);
begin
  if value <> FGridStyle then
  begin
    FGridStyle := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetGridColor(value: TColor);
begin
  if value <> FGridColor then
  begin
    FGridColor := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetTrace1Color(value: TColor);
begin
  if value <> FTrace1Color then
  begin
    FTrace1Color := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetTrace2Color(value: TColor);
begin
  if value <> FTrace2Color then
  begin
    FTrace2Color := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetTrace3Color(value: TColor);
begin
  if value <> FTrace3Color then
  begin
    FTrace3Color := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetTrace4Color(value: TColor);
begin
  if value <> FTrace4Color then
  begin
    FTrace4Color := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetTrace5Color(value: TColor);
begin
  if value <> FTrace5Color then
  begin
    FTrace5Color := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetTrace6Color(value: TColor);
begin
  if value <> FTrace6Color then
  begin
    FTrace6Color := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetTrace7Color(value: TColor);
begin
  if value <> FTrace7Color then
  begin
    FTrace7Color := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetTrace8Color(value: TColor);
begin
  if value <> FTrace8Color then
  begin
    FTrace8Color := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetGridType(value: TGridType);
begin
  if value <> FGridType then
  begin
    FGridType := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetGridSubDiv(value: TGridSubDivType);
begin
  if value <> FGridSubDiv then
  begin
    FGridSubDiv := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetGridSubDivLen(value: word);
begin
  if value <> FGridSubDivLen then
  begin
    FGridSubDivLen := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetLeftMargin(value: integer);
begin
  if value <> FLeftMargin then
  begin
    FLeftMargin := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetRightMargin(value: integer);
begin
  if value <> FRightMargin then
  begin
    FRightMargin := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetTopMargin(value: integer);
begin
  if value <> FTopMargin then
  begin
    FTopMargin := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetBottomMargin(value: integer);
begin
  if value <> FBottomMargin then
  begin
    FBottomMargin := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetHorzMinVal(value: single);
begin
  if value <> FHorzMinVal then
  begin
    FHorzMinVal := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetHorzMaxVal(value: single);
begin
  if value <> FHorzMaxVal then
  begin
    FHorzMaxVal := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetHorzMinText(value: string);
begin
  if value <> FHorzMinText then
  begin
    FHorzMinText := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetHorzMaxText(value: string);
begin
  if value <> FHorzMaxText then
  begin
    FHorzMaxText := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetHorzUnit(value: string);
begin
  if value <> FHorzUnit then
  begin
    FHorzUnit := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetHorzDecNum(value: integer);
begin
  if value <> FHorzDecNum then
  begin
    FHorzDecNum := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetHorzDiv(value: integer);
begin
  if value <> FHorzDiv then
  begin
    FHorzDiv := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetShowVertLimit(value: boolean);
begin
  if value <> FShowVertLimit then
  begin
    FShowVertLimit := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetVertLimitColor(value: TColor);
begin
  if value <> FVertLimitColor then
  begin
    FVertLimitColor := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetVertMaxLimit(value: single);
begin
  if value <> FVertMaxLimit then
  begin
    FVertMaxLimit := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetVertMinLimit(value: single);
begin
  if value <> FVertMinLimit then
  begin
    FVertMinLimit := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetVertMinVal(value: single);
begin
  if value <> FVertMinVal then
  begin
    FVertMinVal := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetVertMaxVal(value: single);
begin
  if value <> FVertMaxVal then
  begin
    FVertMaxVal := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetVertMinText(value: string);
begin
  if value <> FVertMinText then
  begin
    FVertMinText := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetVertMaxText(value: string);
begin
  if value <> FVertMaxText then
  begin
    FVertMaxText := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetVertUnit(value: string);
begin
  if value <> FVertUnit then
  begin
    FVertUnit := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetVertDecNum(value: integer);
begin
  if value <> FVertDecNum then
  begin
    FVertDecNum := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetVertDiv(value: integer);
begin
  if value <> FVertDiv then
  begin
    FVertDiv := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetScaleText(value: TScaleText);
begin
  if value <> FScaleText then
  begin
    FScaleText := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetTraces(value: TTraces);
begin
  if value <> FTraces then
  begin
    FTraces := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetTextColor(value: TColor);
begin
  if value <> FTextColor then
  begin
    FTextColor := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetTextSize(value: integer);
begin
  if value <> FTextSize then
  begin
    FTextSize := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetRangeUse(value: boolean);
begin
  if value <> FRangeUse then
  begin
    FRangeUse := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetRangeMarker(value: boolean);
begin
  if value <> FRangeMarker then
  begin
    FRangeMarker := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetRangeStart(value: integer);
begin
  if value <> FRangeStart then
  begin
    if value >= FRangeStop then
      FRangeStart := 0
    else
      FRangeStart := value;
    OriginPosMarker := -1;
    NewPosMarker := -1;
    Paint;
  end;
end;

procedure TrmAGraph.SetRangeStop(value: integer);
begin
  if value <> FRangeStop then
  begin
    if value <= FRangeStart then
      FRangeStop := 100
    else
      FRangeStop := value;
    OriginPosMarker := -1;
    NewPosMarker := -1;
    Paint;
  end;
end;

procedure TrmAGraph.SetRangeAutoMove(value: boolean);
begin
  if value <> FRangeAutoMove then
  begin
    FRangeAutoMove := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetMarkerColor(value: TColor);
begin
  if value <> FMarkerColor then
  begin
    FMarkerColor := value;
    Paint;
  end;
end;

procedure TrmAGraph.SetMaxItems(value: integer);
var
  i: integer;
begin
  if value <> FMaxItems then
  begin
    FMaxItems := value;
    if (LstValues.Count > FMaxItems) and (FMaxItems > 0) then
      for i := 1 To (LstValues.Count - FMaxItems) do
        LstValues.Delete(0);
    Paint;
  end;
end;

procedure TrmAGraph.Clear;
begin
  LstValues.clear;
  Paint;
end;

function TrmAGraph.AddValues(s: string; refresh: boolean): integer;
var
  i: integer;
begin
  LstValues.add(s);
  if (LstValues.Count > FMaxItems) and (FMaxItems > 0) then
    for i := 1 To (LstValues.Count - FMaxItems) do
      LstValues.Delete(0);
  result := LstValues.count;
  if RangeAutoMove and (valuePosition = vpPixels) then begin
    if lstValues.Count > graphWidth then begin
      FRangeStart := lstValues.Count - graphWidth - 1;
      FRangeStop := lstValues.Count - 1;
    end;
  end;
  if refresh then Paint;
end;

function TrmAGraph.DeleteValues(i: integer; refresh: boolean): integer;
begin
  if i > LstValues.Count - 1 then exit;
  LstValues.delete(i);
  result := LstValues.count;
  if refresh then Paint;
end;

function TrmAGraph.GetValues(x, y: integer; var ValArray: array of real): byte;
var
  s: string;
  i, xPos, sep: integer;
  val: real;
begin
  for i := 0 to 7 do ValArray[i] := 0;
  xPos := x - FLeftMargin + FRangeStart;
  if (xPos > lstValues.Count - 1) or (xPos < 0) then begin
    // showmessage('Out of range !');
    exit;
  end;
  s := LstValues[xPos];
  // if FValuePosition = vpPixels then begin
    for i := 0 to 7 do begin
      sep := pos(';', s);
      if sep > 0 then begin
        val := s2r(copy(s, 1, sep - 1));
        delete(s, 1, sep);
      end else if (sep = 0) and (s <> '') then begin
        val := s2r(s);
        s := '';
      end;
      ValArray[i] := val;
      if s = '' then break;
    end;
  // end;
end;

procedure TrmAGraph.DrawPositionMarker(x: integer);
begin
  if (x > width - FRightMargin) or (x < FLeftMargin) then exit;
  NewPosMarker := x;
  with canvas do begin
    Pen.Mode := pmNotXor;
    Pen.Color := FMarkerColor;
    MoveTo(OriginPosMarker, FTopMargin);
    lineTo(OriginPosMarker, Height - bottomMargin);
    MoveTo(NewPosMarker, FTopMargin);
    lineTo(NewPosMarker, Height - bottomMargin);
    OriginPosMarker := NewPosMarker;
    Pen.Mode := pmCopy;
  end;
end;

procedure TrmAGraph.ClearMarker;
begin
  if OriginPosMarker = -1 then exit;
  with canvas do begin
    Pen.Mode := pmNotXor;
    Pen.Color := FMarkerColor;
    MoveTo(OriginPosMarker, FTopMargin);
    lineTo(OriginPosMarker, Height - bottomMargin);
    OriginPosMarker := -1;
    Pen.Mode := pmCopy;
  end;
end;

function TrmAGraph.GraphBottom: integer;
begin
  result := Height - FBottomMargin;
end;

function TrmAGraph.GraphLeft: integer;
begin
  result := FLeftMargin;
end;

function TrmAGraph.GraphRight: integer;
begin
  result := width - FRightMargin;
end;

function TrmAGraph.GraphTop: integer;
begin
  result := FTopMargin;
end;

function TrmAGraph.GraphHeight: integer;
begin
  result := height - FBottomMargin - FTopMargin;
end;

function TrmAGraph.GraphWidth: integer;
begin
  result := width - FLeftMargin - FRightMargin;
end;

end.

