unit rmBase;

interface

uses
  //Dialogs, Sysutils, // only for show message during debug
  Controls, Classes, ExtCtrls, Graphics;

type

  TrmBorderStyle = (bsNone, bsSingle, bsRaised, bsLowered);

{------------------------------------------------------------------------}
{   Class : TrmBasicShape
    Ancestor : TPersistent (classes.pas unit)
    use : when a simple shape with 3D border is required
{------------------------------------------------------------------------}

  TrmBasicShape = class(TPersistent)
  private
    FParent: TGraphicControl;
    FShape: TShapeType;
    FWidth: integer;
    FHeight: integer;
    FBorderStyle: TrmBorderStyle;
    FBorderWidth: integer;
    FBackGroundColor: TColor;
    FBorderColor: TColor;
    FActiveColor: TColor;
    FInactiveColor: TColor;
    FStandByColor: TColor;
    FHilightColor: TColor;
    FShaddowColor: TColor;
    FActive: boolean;
    FStandBy: boolean;
    FVisible: boolean;
    procedure SetShape(value: TShapeType);
    procedure SetWidth(value: integer);
    procedure SetHeight(value: integer);
    procedure SetBorderStyle(value: TrmBorderStyle);
    procedure SetBorderWidth(value: integer);
    procedure SetBorderColor(value: TColor);
    procedure SetBackGroundColor(value: TColor);
    procedure SetActiveColor(value: TColor);
    procedure SetInactiveColor(value: TColor);
    procedure SetStandByColor(value: TColor);
    procedure SetHilightColor(value: TColor);
    procedure SetShaddowColor(value: TColor);
    procedure SetActive(value: boolean);
    procedure SetStandBy(value: boolean);
    procedure SetVisible(value: boolean);
  protected
    procedure Paint; //override;
    procedure UpdateBmpBuffer;
  public
    FRefreshParentAfterChange: boolean;
    FBmpBuffer: TBitmap;
    constructor Create(Owner: TGraphicControl); //override;
    destructor destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); //override;
    property Shape: TShapeType read FShape write SetShape;
    property Width: integer read FWidth write SetWidth;
    property Height: integer read FHeight write SetHeight;
    property BorderStyle: TrmBorderStyle read FBorderStyle write SetBorderStyle;
    property BorderWidth: integer read FBorderWidth write SetBorderWidth;
    property BackGroundColor: TColor read FBackGroundColor write SetBackGroundColor;
    property BorderColor: TColor read FBorderColor write SetBorderColor;
    property ActiveColor: TColor read FActiveColor write SetActiveColor;
    property InactiveColor: TColor read FInactiveColor write SetInactiveColor;
    property StandByColor: TColor read FStandByColor write SetStandByColor;
    property HilightColor: TColor read FHilightColor write SetHilightColor;
    property ShaddowColor: TColor read FShaddowColor write SetShaddowColor;
    property Active: boolean read FActive write SetActive;
    property StandBy: boolean read FStandBy write FStandBy;
    property Visible: boolean read FVisible write SetVisible;
  published
  end;

{------------------------------------------------------------------------}
{   Class : TrmBackGround
    Ancestor : TPersistent (classes.pas unit)
    use : when a simple background with 3D border is required
{------------------------------------------------------------------------}

  TrmBackGround = class(TPersistent)
  private
    FParent: TGraphicControl;
    FWidth: integer;
    FHeight: integer;
    FBorderStyle: TrmBorderStyle;
    FBackGroundColor: TColor;
    FBorderColor: TColor;
    FBorderWidth: integer;
    FHilightColor: TColor;
    FShaddowColor: TColor;
    procedure SetWidth(value: integer);
    procedure SetHeight(value: integer);
    procedure SetBorderStyle(value: TrmBorderStyle);
    procedure SetBorderWidth(value: integer);
    procedure SetBorderColor(value: TColor);
    procedure SetBackGroundColor(value: TColor);
    procedure SetHilightColor(value: TColor);
    procedure SetShaddowColor(value: TColor);
  protected
    procedure Paint; //override;
    procedure UpdateBmpBuffer;
  public
    FRefreshParentAfterChange: boolean;
    FBmpBuffer: TBitmap;
    constructor Create(Owner: TGraphicControl); //override;
    destructor destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); //override;
    property Width: integer read FWidth write SetWidth;
    property Height: integer read FHeight write SetHeight;
    property BorderStyle: TrmBorderStyle read FBorderStyle write SetBorderStyle;
    property BorderWidth: integer read FBorderWidth write SetBorderWidth;
    property BackGroundColor: TColor read FBackGroundColor write SetBackGroundColor;
    property BorderColor: TColor read FBorderColor write SetBorderColor;
    property HilightColor: TColor read FHilightColor write SetHilightColor;
    property ShaddowColor: TColor read FShaddowColor write SetShaddowColor;
  published
  end;


implementation

{------------------------------------------------------------------------}
{   TrmBasicShape
{------------------------------------------------------------------------}

constructor TrmBasicShape.Create(Owner: TGraphicControl);
begin
  Inherited Create;
  FBmpBuffer := TBitmap.create;
  FParent := Owner;
  FRefreshParentAfterChange := true;
  FShape := stRectangle;
  FWidth := 13;
  FHeight := 13;
  FBorderStyle := bsLowered;
  FBorderColor := clBlue;
  FBackgroundColor := clSilver;
  FActiveColor := clLime;
  FInactiveColor := clGray;
  FStandByColor := clSilver;
  FHilightColor := clWhite;
  FShaddowColor := clGray;
  FBorderWidth := 1;
  FActive := true;
  FStandBy := false;
  FVisible := true;
  SetBounds(0, 0, width, height);
  UpdateBmpBuffer;
end;

destructor TrmBasicShape.Destroy;
begin
  FBmpBuffer.free;
  Inherited;
end;

procedure TrmBasicShape.SetBounds(aLeft, aTop, aWidth, aHeight: Integer);
begin
  Inherited ;
  If FBmpBuffer <> NIL then begin
    FBmpBuffer.Destroy;
    FBmpBuffer := TBitmap.Create;
    FBmpBuffer.Height := aHeight;
    FBmpBuffer.Width := aWidth;
  end;
end;

procedure TrmBasicShape.UpdateBmpBuffer;

  procedure DrawBackGround;
  begin
    With FBmpBuffer.Canvas do begin

      pen.Width := BorderWidth;

      // if not shape is rectangle then draw background
      if not (FShape = stRectangle) then begin
        Brush.Color := FBackGroundColor;
        Pen.Color := FBackGroundColor;
        rectangle(0, 0, width, height);
      end;

      if FStandBy then begin
        Brush.Color := FStandByColor;
        if FBorderStyle = bsSingle then
          Pen.Color := FBorderColor
        else
          Pen.Color := FStandByColor;
      end else begin
        if FActive then begin
          Brush.Color := FActiveColor;
          if FBorderStyle = bsSingle then
            Pen.Color := FBorderColor
          else
            Pen.Color := FActiveColor;
        end else begin
          Brush.Color := FInactiveColor;
          if FBorderStyle = bsSingle then
            Pen.Color := FBorderColor
          else
            Pen.Color := FInactiveColor;
        end;
      end;

      case FShape of
        stRectangle : rectangle(0, 0, width, height);
        stRoundRect : roundRect(0, 0, width, height, 4, 4);
        stEllipse   : ellipse(0, 0, width, height);
      end;
    end;
  end;

  procedure Draw3DBorder;
  var
    i: integer;
  begin
    With FBmpBuffer.Canvas do begin

      pen.Width := 1;

      for i := 0 to FBorderWidth - 1 do begin

        if FBorderStyle = bsRaised then
          Pen.Color := FHilightColor
        else
          Pen.Color := FShaddowColor;

        MoveTo(i, height - 1 - i);
        LineTo(i, i);
        LineTo(width - 1 - i, i);

        if FBorderStyle = bsRaised then
          Pen.Color := FShaddowColor
        else
          Pen.Color := FHilightColor;

        LineTo(width - 1 - i, height - 1 - i);
        LineTo(i, height - 1 - i);
      end;

    end;
  end;

begin
  DrawBackGround;
  if (FBorderStyle = bsRaised) or (FBorderStyle = bsLowered) then
    Draw3DBorder;

end;

procedure TrmBasicShape.Paint;
begin
  UpdateBmpBuffer;
  //FParent.Repaint;
  //FParent.Canvas.Draw(0, 0, FBmpBuffer);
end;

procedure TrmBasicShape.SetShape(value: TShapeType);
begin
  if (value <> stEllipse) and (value <> stRoundRect) and (value <> stRectangle) then exit;
  if value <> FShape then
  begin
    FShape := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBasicShape.SetWidth(value: integer);
begin
  if value <> FWidth then
  begin
    FWidth := value;
    SetBounds(0, 0, width, height);
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBasicShape.SetHeight(value: integer);
begin
  if value <> FHeight then
  begin
    FHeight := value;
    SetBounds(0, 0, width, height);
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBasicShape.SetBorderStyle(value: TrmBorderStyle);
begin
  if value <> FBorderStyle then
  begin
    FBorderStyle := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBasicShape.SetBorderWidth(value: integer);
begin
  if value <> FBorderWidth then
  begin
    FBorderWidth := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBasicShape.SetBorderColor(value: TColor);
begin
  if value <> FBorderColor then
  begin
    FBorderColor := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBasicShape.SetBackGroundColor(value: TColor);
begin
  if value <> FBackGroundColor then
  begin
    FBackGroundColor := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBasicShape.SetActiveColor(value: TColor);
begin
  if value <> FActiveColor then
  begin
    FActiveColor := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBasicShape.SetInactiveColor(value: TColor);
begin
  if value <> FInactiveColor then
  begin
    FInactiveColor := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBasicShape.SetStandByColor(value: TColor);
begin
  if value <> FStandByColor then
  begin
    FStandByColor := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBasicShape.SetHilightColor(value: TColor);
begin
  if value <> FHilightColor then
  begin
    FHilightColor := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBasicShape.SetShaddowColor(value: TColor);
begin
  if value <> FShaddowColor then
  begin
    FShaddowColor := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBasicShape.SetActive(value: boolean);
begin
  if value <> FActive then
  begin
    FActive := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBasicShape.SetVisible(value: boolean);
begin
  if value <> FVisible then
  begin
    FVisible := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBasicShape.SetStandBy(value: boolean);
begin
  if value <> FStandBy then
  begin
    FStandBy := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

{------------------------------------------------------------------------}
{   TrmBackGround
{------------------------------------------------------------------------}

constructor TrmBackGround.Create(Owner: TGraphicControl);
begin
  Inherited Create;
  FBmpBuffer := TBitmap.create;
  FParent := Owner;
  FRefreshParentAfterChange := true;
  FWidth := 13;
  FHeight := 13;
  FBorderStyle := bsLowered;
  FBorderColor := clBlue;
  FBackgroundColor := clSilver;
  FHilightColor := clWhite;
  FShaddowColor := clGray;
  FBorderWidth := 1;
  SetBounds(0, 0, width, height);
  UpdateBmpBuffer;
end;

destructor TrmBackGround.Destroy;
begin
  FBmpBuffer.free;
  Inherited;
end;

procedure TrmBackGround.SetBounds(aLeft, aTop, aWidth, aHeight: Integer);
begin
  Inherited ;
  If FBmpBuffer <> NIL then begin
    FBmpBuffer.Destroy;
    FBmpBuffer := TBitmap.Create;
    FBmpBuffer.Height := aHeight;
    FBmpBuffer.Width := aWidth;
  end;
end;

procedure TrmBackGround.UpdateBmpBuffer;

  procedure DrawBackGround;
  begin
    With FBmpBuffer.Canvas do begin
      pen.Width := BorderWidth;
      Brush.Color := FBackGroundColor;
      Pen.Color := FBackGroundColor;
      rectangle(0, 0, width, height);
    end;
  end;

  procedure Draw3DBorder;
  var
    i: integer;
  begin
    With FBmpBuffer.Canvas do begin

      pen.Width := 1;

      for i := 0 to FBorderWidth - 1 do begin

        if FBorderStyle = bsRaised then
          Pen.Color := FHilightColor
        else
          Pen.Color := FShaddowColor;

        MoveTo(i, height - 1 - i);
        LineTo(i, i);
        LineTo(width - 1 - i, i);

        if FBorderStyle = bsRaised then
          Pen.Color := FShaddowColor
        else
          Pen.Color := FHilightColor;

        LineTo(width - 1 - i, height - 1 - i);
        LineTo(i, height - 1 - i);
      end;

    end;
  end;

begin
  DrawBackGround;
  if (FBorderStyle = bsRaised) or (FBorderStyle = bsLowered) then
    Draw3DBorder;

end;

procedure TrmBackGround.Paint;
begin
  UpdateBmpBuffer;
  //FParent.Repaint;
  //FParent.Canvas.Draw(0, 0, FBmpBuffer);
end;

procedure TrmBackGround.SetWidth(value: integer);
begin
  if value <> FWidth then
  begin
    FWidth := value;
    SetBounds(0, 0, width, height);
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBackGround.SetHeight(value: integer);
begin
  if value <> FHeight then
  begin
    FHeight := value;
    SetBounds(0, 0, width, height);
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBackGround.SetBorderStyle(value: TrmBorderStyle);
begin
  if value <> FBorderStyle then
  begin
    FBorderStyle := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBackGround.SetBorderWidth(value: integer);
begin
  if value <> FBorderWidth then
  begin
    FBorderWidth := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBackGround.SetBorderColor(value: TColor);
begin
  if value <> FBorderColor then
  begin
    FBorderColor := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBackGround.SetBackGroundColor(value: TColor);
begin
  if value <> FBackGroundColor then
  begin
    FBackGroundColor := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBackGround.SetHilightColor(value: TColor);
begin
  if value <> FHilightColor then
  begin
    FHilightColor := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;

procedure TrmBackGround.SetShaddowColor(value: TColor);
begin
  if value <> FShaddowColor then
  begin
    FShaddowColor := value;
    UpdateBmpBuffer;
    if FRefreshParentAfterChange then
      FParent.Repaint;
  end;
end;



end.
