unit rmSlider;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls;

type
  TrmSlider = class(TCustomControl)
  private
    Thumb: TRect;
    MemDC: HDC;
    Bitmap: HBitmap;
    Capture: boolean;
    CapturePoint: TPoint;
    CaptureValue: Integer;

    FTrackWidth: Integer;
    FTrackExtremityOffset: integer;
    FThumbHeight: Integer;
    FThumbCenterLine: boolean;
    FThumbCenterLineColor: TColor;
    FMinValue: Integer;
    FMaxValue: Integer;
    FPosition: Integer;
    FChange: TNotifyEvent;

    procedure SetTrackWidth(value: Integer);
    procedure SetTrackExtremityOffset(value: Integer);
    procedure SetThumbHeight(value: Integer);
    procedure SetThumbCenterLine(value: boolean);
    procedure SetThumbCenterLineColor(value: TColor);
    procedure SetMinValue(v: Integer);
    procedure SetMaxValue(v: Integer);
    procedure SetPosition(value: Integer);

  protected
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure DrawThumb; virtual;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  published
    property TrackWidth: Integer read fTrackWidth write SetTrackWidth;
    property TrackExtremityOffset: Integer read FTrackExtremityOffset write SetTrackExtremityOffset;
    property ThumbHeight: Integer read fThumbHeight write SetThumbHeight;
    property ThumbCenterLine: boolean read FThumbCenterLine write SetThumbCenterLine;
    property ThumbCenterLineColor: TColor read FThumbCenterLineColor write SetThumbCenterLineColor;
    property MinValue: Integer read FMinValue write SetMinValue;
    property MaxValue: Integer read FMaxValue write SetMaxValue;
    property Position: Integer read FPosition write SetPosition;
    property OnChange: TNotifyEvent read fChange write fChange;

    property Color;
    property Enabled;
    property HelpContext;
    property Hint;
    property ParentShowHint;
    property ShowHint;
    property PopupMenu;
    property Tag;
    property Visible;

    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;

  end;

procedure Register;

implementation

constructor TrmSlider.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  Width := 16;
  Height := 145;
  FTrackWidth := 3;
  FTrackExtremityOffset := 10;
  FMinValue := 0;
  FMaxValue := 100;
  FPosition := 0;
  FThumbHeight := 32;
  FThumbCenterLine := true;
  FThumbCenterLineColor := clRed;
  FChange := Nil;
  Capture := False;
  Thumb.left := -1;
end;

destructor TrmSlider.Destroy;
begin
  if Bitmap <> 0 then DeleteObject(Bitmap);
  if MemDC <> 0 then DeleteDC(MemDC);
  inherited Destroy;
end;

procedure TrmSlider.SetTrackWidth(value : Integer);
begin
  if fTrackWidth <> value then
  begin
    fTrackWidth := value;
    paint;
    Invalidate;
  end
end;

procedure TrmSlider.SetTrackExtremityOffset(value : Integer);
begin
  if FTrackExtremityOffset <> value then
  begin
    FTrackExtremityOffset := value;
    paint;
    Invalidate;
  end
end;

procedure TrmSlider.SetThumbHeight(value : Integer);
begin
  if value <> fThumbHeight then
  begin
    fThumbHeight := value;
    paint;
    Invalidate;
  end
end;

procedure TrmSlider.SetThumbCenterLineColor(value : TColor);
begin
  if value <> FThumbCenterLineColor then
  begin
    FThumbCenterLineColor := value;
    paint;
    Invalidate;
  end
end;

procedure TrmSlider.SetThumbCenterLine(value : boolean);
begin
  if value <> FThumbCenterLine then
  begin
    FThumbCenterLine := value;
    paint;
    Invalidate;
  end
end;

procedure TrmSlider.SetMinValue(v: Integer);
begin
  if v <> fMinValue then
  begin
    if ((v = fMaxValue) or (v < 0)) then
      fMinValue := 0
    else
      fMinValue := V;
    if Position < fMinValue then
      Position := fMinValue;
    paint;
    Invalidate;
  end
end;

procedure TrmSlider.SetMaxValue(v: Integer);
begin
  if v <> fMaxValue then
  begin
    if ((v > 100) or (v = fMinValue)) then
      fMaxValue := 100
    else
      fMaxValue := V;
    if Position > fMaxValue then
      Position := fMaxValue;
    paint;
    Invalidate;
  end
end;

procedure TrmSlider.SetPosition(value: Integer);
begin
  if value < MinValue then
    value := MinValue
  else if value > MaxValue then
    value := MaxValue;
  if value <> fPosition then
  begin
    fPosition := Value;
    if Assigned(fChange) then
      OnChange(self);
    DrawThumb;
    if csDesigning in componentState then
      paint;
      Invalidate;
    end
end;

procedure TrmSlider.Paint;
var
  Rect: TRect;
begin
  with Canvas do begin

    // Create memory DC for save bitmap
    if MemDC = 0 then MemDC := CreateCompatibleDC (Canvas.Handle);

    // Create thumb & bitmap for vertical slider
    if Bitmap = 0 then
      Bitmap := CreateCompatibleBitmap(Canvas.Handle, Width, ThumbHeight);
    Rect.top := 0;
    Rect.bottom := Height;
    Rect.left := (Width - TrackWidth) div 2;
    Rect.Right := Rect.Left + TrackWidth;

    // Draw track
    Brush.Color := clBlack;
    Pen.Color := clBtnHighlight;
    with Rect do begin
      Rectangle (left, top + FTrackExtremityOffset, right, bottom - FTrackExtremityOffset);
      Pen.Color := clBtnShadow;
      MoveTo (left, top + FTrackExtremityOffset);
      LineTo (right, top + FTrackExtremityOffset);
      MoveTo (left, top + FTrackExtremityOffset);
      LineTo (left, bottom - FTrackExtremityOffset)
    end;

    // draw thumb
    DrawThumb;

  end
end;

// Draw the thumb at the correct position for the current value.           |
procedure TrmSlider.DrawThumb;

  procedure DrawThumb;
  begin
    with Canvas, Thumb do begin
      // thumb top half, and thumb bottom half
      Pen.Color := clGray;   // clBlack
      Brush.Color := $00B0B0B0; // a little darker than clSilver;
      Rectangle (left, top, right - 1, bottom - (thumbHeight div 2));
      Brush.Color := clSilver;
      Rectangle (left, bottom - (thumbHeight div 2), right - 1, bottom - 1);
    end;
  end;

  procedure DrawThumb3D;
  begin
    with Canvas, Thumb do begin
      // 3D Effect on border
      Pen.Color := clWhite;
      MoveTo(Left + 1, Bottom - 3);
      LineTo(Left + 1, Top + 1);
      MoveTo(Left + 1, Top + 1);
      LineTo(Right - 3, Top + 1);
    end;
  end;

  procedure DrawThumbCenterLine;
  begin
    with Canvas, Thumb do begin
      if not FThumbCenterLine then exit;
      Pen.Color := FThumbCenterLineColor;
      MoveTo(Left + 2, Bottom - (thumbHeight div 2));
      LineTo(Right - 3, Bottom - (thumbHeight div 2));
      Pen.Color := clBlack;
      MoveTo(Left + 2, Bottom - (thumbHeight div 2) - 1);
      LineTo(Right - 3, Bottom - (thumbHeight div 2) - 1);
      Pen.Color := clWhite;
      MoveTo(Left + 2, Bottom - (thumbHeight div 2) + 1);
      LineTo(Right - 3, Bottom - (thumbHeight div 2) + 1);
    end;
  end;

var
  basePos: Integer;
  rc: bool;
  oldBmp: HBitmap;
  oldThumb: TRect;
begin
  if csLoading in ComponentState then Exit;

  oldBmp := SelectObject(MemDC, Bitmap);

  // Set thumb color & border color
  Canvas.Brush.Color := clBlue;
  Canvas.Pen.Color := clBtnHighlight;

  // Save current thum rectangle
  oldThumb := Thumb;

  // Calculate new thumb rectangle for vert slider
  basePos := LongInt(Height - ThumbHeight) * LongInt(Position - MinValue) div (MaxValue - MinValue);
  Thumb.left := 0;
  Thumb.right := Width;
  Thumb.Bottom := Height - BasePos;
  Thumb.top := Thumb.Bottom - ThumbHeight;

  // Rub out old thumb
  if oldThumb.left <> -1 then
    with oldThumb do
      BitBlt(Canvas.Handle, Left, Top, Width, ThumbHeight, MemDC, 0, 0, SRCCOPY);

  // Save what's underneath
  with Thumb do
    rc := BitBlt(MemDC, 0, 0, Width, ThumbHeight, Canvas.Handle, Left, Top, SRCCOPY);

  // Draw thumb
  DrawThumb;
  DrawThumb3D;
  DrawThumbCenterLine;

  SelectObject(MemDC, OldBmp);
end;

procedure TrmSlider.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if (Button = mbLeft) and PtInRect(Thumb, Point(X, Y)) then
  begin
    capture := True;
    capturePoint := Point(X, Y);
    captureValue := Position;
  end;
end;

procedure TrmSlider.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  if (Button = mbLeft) then capture := False
end;

procedure TrmSlider.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseMove (shift, X, Y);
  if capture then
    Position := captureValue + MinValue + LongInt(MaxValue - MinValue) * LongInt(capturePoint.Y - Y) div (Height - ThumbHeight)
end;

procedure Register;
begin
  RegisterComponents('OptoElec', [TrmSlider]);
end;

end.
