unit rmZGraph;

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs,  ExtCtrls, clipbrd;

type
  TGridType = (gtNone, gtVertical, gtHorizontal, gtBoth);
  TBorderType = (btNone, btRaised, btLowered, btSingle);
  TGridSubDivType = (sdNone, sdVertical, sdHorizontal, sdBoth);
  TTextType = (stUseHMinMax, stUseVMinMax, stLeft, stRight, stBottom, stTop, stHorzOnlyExtremity, stVertOnlyExtremity);
  TScaleText = set of TTextType;

  TrmZGraph = class(TGraphicControl)
  private
    FBmpBuffer: TBitmap;
    FGridType: TGridType;
    FGridSubDiv: TGridSubDivType;
    FGridSubDivLen: word;
    FCTL3D: boolean;
    FBGColor: TColor;
    FBorder: TBorderType;
    FGraphColor: TColor;
    FGridStyle: TPenStyle;
    FGridColor: TColor;
    FLeftMargin, FRightMargin, FTopMargin, FBottomMargin: integer;
    FHorzMinVal, FVertMinVal: real;
    FHorzMaxVal, FVertMaxVal: real;
    FHorzMinText, FVertMinText: string;
    FHorzMaxText, FVertMaxText: string;
    FHorzUnit, FVertUnit: string;
    FHorzDecNum, FVertDecNum: integer;
    FHorzDiv, FVertDiv: integer;
    FScaleText: TScaleText;
    FTextColor: TColor;
    FTextSize: integer;
    FMaxItems: integer;
    function  DecodeLine(StrToDec: string; var sArray: array of string): boolean;
    procedure SetGridType(value: TGridType);
    procedure SetGridSubDiv(value: TGridSubDivType);
    procedure SetGridSubDivLen(value: word);
    procedure SetCTL3D(value: boolean);
    procedure SetLstValues(value: TStringList);
    procedure SetBGColor(value: TColor);
    procedure SetBorder(value: TBorderType);
    procedure SetGraphColor(value: TColor);
    procedure SetGridStyle(value: TPenStyle);
    procedure SetGridColor(value: TColor);
    procedure SetLeftMargin(value: integer);
    procedure SetRightMargin(value: integer);
    procedure SetTopMargin(value: integer);
    procedure SetBottomMargin(value: integer);
    procedure SetHorzMinVal(value: real);
    procedure SetHorzMaxVal(value: real);
    procedure SetHorzMinText(value: string);
    procedure SetHorzMaxText(value: string);
    procedure SetHorzUnit(value: string);
    procedure SetHorzDecNum(value: integer);
    procedure SetHorzDiv(value: integer);
    procedure SetVertMinVal(value: real);
    procedure SetVertMaxVal(value: real);
    procedure SetVertMinText(value: string);
    procedure SetVertMaxText(value: string);
    procedure SetVertUnit(value: string);
    procedure SetVertDecNum(value: integer);
    procedure SetVertDiv(value: integer);
    procedure SetScaleText(value: TScaleText);
    procedure SetTextColor(value: TColor);
    procedure SetTextSize(value: integer);
    procedure SetMaxItems(value: integer);
  protected
    procedure Paint; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure UpdateBmpBuffer(colored: boolean);
  public
    constructor Create(anOwner:TComponent); override;
    destructor Destroy; override;
    procedure Free;
  published
    LstValues: TStringList;
    property GridType: TGridType read FGridType write SetGridType;
    property GridSubDiv: TGridSubDivType read FGridSubDiv write SetGridSubDiv;
    property GridSubDivLen: word read FGridSubDivLen write SetGridSubDivLen;
    // property Values: TStringList read FValues write SetValues;
    property BGColor: TColor read FBGColor write SetBGColor;
    property CTL3D: boolean read FCTL3D write SetCTL3D;
    property Border: TBorderType read FBorder write SetBorder;
    property GraphColor: TColor read FGraphColor write SetGraphColor;
    property GridStyle: TPenStyle read FGridStyle write SetGridStyle;
    property GridColor: TColor read FGridColor write SetGridColor;
    property LeftMargin: integer read FLeftMargin write SetLeftMargin;
    property RightMargin: integer read FRightMargin write SetRightMargin;
    property TopMargin: integer read FTopMargin write SetTopMargin;
    property BottomMargin: integer read FBottomMargin write SetBottomMargin;
    property HorzMinVal: real read FHorzMinVal write SetHorzMinVal;
    property HorzMaxVal: real read FHorzMaxVal write SetHorzMaxVal;
    property HorzMinText: string read FHorzMinText write SetHorzMinText;
    property HorzMaxText: string read FHorzMaxText write SetHorzMaxText;
    property HorzUnit: string read FHorzUnit write SetHorzUnit;
    property HorzDiv: integer read FHorzDiv write SetHorzDiv;
    property HorzDecNum: integer read FHorzDecNum write SetHorzDecNum;
    property VertMinVal: real read FVertMinVal write SetVertMinVal;
    property VertMaxVal: real read FVertMaxVal write SetVertMaxVal;
    property VertMinText: string read FVertMinText write SetVertMinText;
    property VertMaxText: string read FVertMaxText write SetVertMaxText;
    property VertUnit: string read FVertUnit write SetVertUnit;
    property VertDecNum: integer read FVertDecNum write SetVertDecNum;
    property VertDiv: integer read FVertDiv write SetVertDiv;
    property ScaleText: TScaleText read FScaleText write SetScaleText;
    property TextColor: TColor read FTextColor write SetTextColor;
    property TextSize: integer read FTextSize write SetTextSize;
    property MaxItems: integer read FMaxItems write SetMaxItems;

    procedure LoadDataFromFile(sFile: TFileName);
    procedure SaveDataToFile(sFile: TFileName);
    procedure SaveGraphToFile(colored: boolean; sFile: TFileName);
    procedure CopyGraphToClipboard(colored: boolean);
    procedure Clear;
    function  AddValues(s: string; refresh: boolean): integer;
    function  DeleteValues(i: integer; refresh: boolean): integer;
    function  GetValues(x, y: integer; var ValArray: array of real): byte;
    function  GetHorzValueFromXPos(x: integer): real;
    function  GetVertValueFromYPos(y: integer): real;

    function GraphBottom: integer;
    function GraphLeft: integer;
    function GraphRight: integer;
    function GraphTop: integer;
    function GraphHeight: integer;
    function GraphWidth: integer;

    { Standard properties }
    property Align;
    property Height;
    property Width;
    property PopupMenu;
    property Hint;

    { Standard events }
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('OptoElec', [TrmZGraph]);
end;

function r2s(dec: integer; r: real): string;
var
  s: string;
begin
  str(r:0:dec, s);
  result := s;
end;

function s2r(s: string): real;
var
  r: real;
  u: integer;
begin
  val(s, r, u);
  result := r;
end;

function TrmZGraph.DecodeLine(StrToDec: string; var sArray: array of string): boolean;
var
  i, sep, cnt: integer;
  s, sTmp: string;
begin
  result := false;
  cnt := 0;

  // empty array of string
  for i := 0 to High(sArray) do
    sArray[i] := '';

  // if string to decode empty, exit function
  if StrToDec = '' then exit;

  // else continue via temp string
  sTmp := trim(StrToDec);

  // if only one value to extract (for color affectation for exemple)...
  if (pos(':=', sTmp) > 0) and not((pos('(', sTmp) > 0) and (pos(')', sTmp) > 0)) then begin

    // first, get ':=' affectation symbol position
    sep := pos(':=', sTmp);

    // extract command text or property (at the left of ':=' affectation symbol)
    s := copy(sTmp, 1, sep - 1);
    sArray[cnt] := upperCase(trim(s));
    inc(cnt);

    // now, work on the value part (at the right of ':=' affectation symbol)
    delete(sTmp, 1, sep + 1);
    s := trim(sTmp);
    sArray[cnt] := upperCase(trim(s));
    inc(cnt);

  // if multiple values to extract (pos cursor for exemple)...
  // end else if (not pos(':=', sTmp) > 0) and ((pos('(', sTmp) > 0) and (pos(')', sTmp) > 0)) then begin
  end else begin

    // first, get '(' symbol position
    sep := pos('(', sTmp);

    // extract command text (at the left of '(' symbol)
    s := copy(sTmp, 1, sep - 1);
    sArray[cnt] := upperCase(trim(s));
    inc(cnt);

    // now, work on the values part (at the right of '(' symbol and at the left of ')' symbol)
    delete(sTmp, 1, sep);              // delete from begin of string to '('
    delete(sTmp, length(sTmp), 1);     // delete ')' (last char)
    s := trim(sTmp);

    // search each occurence of ',' separator
    while (pos(',', sTmp) > 0) do begin
      sep := pos(',', sTmp);
      s := copy(sTmp, 1, sep - 1);
      sArray[cnt] := upperCase(trim(s));
      delete(sTmp, 1, sep);
      inc(cnt);
    end;
    if sTmp <> '' then
      sArray[cnt] := upperCase(trim(sTmp));

  end;
end;

constructor TrmZGraph.Create(anOwner:TComponent);
begin
  Inherited Create(anOwner);
  FBmpBuffer := TBitmap.Create;
  LstValues := TStringList.Create;
  FBorder := btRaised;
  FCTL3D := true;
  FBGColor := clSilver;
  FGraphColor := clBlack;
  FGridStyle := psDot;
  FGridColor := clGreen;
  Height := 305;
  Width := 433;
  FGridType := gtBoth;
  FGridSubDiv := sdNone;
  FGridSubDivLen := 3;
  FLeftMargin := 45;
  FRightMargin := 4;
  FTopMargin := 4;
  FBottomMargin := 20;
  FHorzMinVal := 0;
  FHorzMaxVal := 100;
  FHorzMinText := 'HMin';
  FHorzMaxText := 'HMax';
  FHorzUnit := '';
  FHorzDecNum := 0;
  FHorzDiv := 10;
  FVertMinVal := 0;
  FVertMaxVal := 100;
  FVertMinText := 'VMin';
  FVertMaxText := 'VMax';
  FVertUnit := '';
  FVertDecNum := 1;
  FVertDiv := 10;
  FScaleText := [stUseVMinMax, stLeft, stBottom];
  FTextColor := clBlue;
  FTextSize := 8;
  FMaxItems := 0;
end;

destructor TrmZGraph.Destroy;
begin
  If FBmpBuffer <> NIL then FBmpBuffer.free;
  if LstValues <> nil then LstValues.Free;
  Inherited Destroy;
end;

procedure TrmZGraph.Free;
begin
  FBmpBuffer.Free;
  LstValues.free;
  Inherited Free;
end;

procedure TrmZGraph.SetBounds(aLeft, aTop, aWidth, aHeight: Integer);
begin
  Inherited ;
  If FBmpBuffer <> NIL then begin
    FBmpBuffer.Destroy;
    FBmpBuffer := TBitmap.Create;
    FBmpBuffer.Height := aHeight;
    FBmpBuffer.Width := aWidth;
  end;
end;

procedure TrmZGraph.UpdateBmpBuffer(colored: boolean);

  procedure DrawBackGround;
  begin
    With FBmpBuffer.Canvas do begin
      if colored then begin
        if (FBorder <> btSingle) then Pen.Color := FBGColor
        else Pen.Color := clBlack;
        Brush.Color := FBGColor;
      end else begin
        if (FBorder <> btNone) then Pen.Color := clBlack
        else pen.color := clWhite;
        Brush.Color := clWhite;
      end;
      Rectangle(0, 0, width, height);

      // 3D border only if colored
      if colored then begin
        if (FBorder = btRaised) or (FBorder = btLowered) then begin
          if FBorder = btRaised then Pen.Color := clWhite else pen.color := clGray;
          moveTo(0, height - 1);
          lineTo(0, 0);
          lineTo(width - 1, 0);
          if FBorder = btRaised then Pen.Color := clGray else pen.color := clWhite;
          lineTo(width - 1, height - 1);
          lineTo(0, height - 1);
        end;
      end;
    end;
  end;

  procedure DrawGraphBackGround;
  var
    GraphW, GraphH: integer;
  begin
    graphW := GraphWidth;
    graphH := GraphHeight;
    With FBmpBuffer.Canvas do begin
      if colored then begin
        Pen.Color := FGraphColor;
        Brush.Color := FGraphColor;
      end else begin
        Pen.Color := clBlack;
        Brush.Color := clWhite;
      end;
      Rectangle(FLeftMargin, FTopMargin, width - FRightMargin, Height - bottomMargin);

      // 3D border on draw zone only if colored
      if colored and FCTL3D then begin
        Pen.Color := clGray;
        moveTo(leftMargin - 1, TopMargin + graphH);
        lineTo(leftMargin - 1, TopMargin - 1);
        lineTo(leftMargin + graphW, TopMargin - 1);
        Pen.Color := clWhite;
        lineTo(leftMargin + graphW, TopMargin  + graphH);
        lineTo(leftMargin - 1, TopMargin + graphH);
      end;
    end;
  end;

  procedure DrawGrid;
  var
    i: integer;
    HPos, VPos: integer;
    GraphW, GraphH: integer;
  begin
    graphW := GraphWidth;
    graphH := GraphHeight;
    With FBmpBuffer.Canvas do begin
      pen.style := FGridStyle;
      if colored then Pen.Color := FGridColor else Pen.Color := clBlack;
      if (FGridType = gtHorizontal) or (FGridType = gtBoth) then begin
        for i := 1 to FHorzDiv - 1 do begin
          HPos := FLeftMargin + trunc((i * GraphW) / FHorzDiv);
          MoveTo(HPos, FTopMargin);
          LineTo(HPos, Height - FBottomMargin);
        end;
        // Horizontal Sub-divisions
        if (FGridSubDiv = sdHorizontal) or (FGridSubDiv = sdBoth) then begin
          for i := 1 to (FHorzDiv * 10) - 1 do begin
            HPos := FLeftMargin + trunc((i * GraphW) / (FHorzDiv * 10));
            MoveTo(HPos, Height - FBottomMargin - FGridSubDivLen);
            LineTo(HPos, Height - FBottomMargin);
          end;
        end;
      end;
      if (FGridType = gtVertical) or (FGridType = gtBoth) then begin
        for i := 1 to FVertDiv - 1 do begin
          VPos := FTopMargin + trunc((i * GraphH) / FVertDiv);
          MoveTo(FLeftMargin, VPos);
          LineTo(width - FRightMargin, VPos);
        end;
        // Vertical Sub-divisions
        if (FGridSubDiv = sdVertical) or (FGridSubDiv = sdBoth) then begin
          for i := 1 to (FVertDiv * 10) - 1 do begin
            VPos := FTopMargin + trunc((i * GraphH) / (FVertDiv * 10));
            MoveTo(FLeftMargin, VPos);
            LineTo(FLeftMargin + FGridSubDivLen, VPos);
          end;
        end;
      end;
      pen.style := psSolid;
    end;
  end;

  procedure DrawScaleText;
  var
    i: integer;
    tmpVal: real;
    s: string;
    HPos, VPos: integer;
    GraphH, GraphW: integer;
  begin
    graphW := GraphWidth;
    graphH := GraphHeight;
    With FBmpBuffer.Canvas do begin
      // left, right, top and bottom scale text
      if (stLeft in FScaleText) or (stRight in FScaleText) then begin
        font.Name := 'Arial';
        font.Size := FTextSize;
        if colored then begin
          brush.Color := FBGColor;
          font.Color := FTextColor;
        end else begin
          brush.Color := clWhite;
          font.Color := clBlack;
        end;
        if (stUseVMinMax in FScaleText) then begin
          for i := 0 to FVertDiv do begin
            if (i <> 0) and (i <> FVertDiv) and (stVertOnlyExtremity in FScaleText) then
              continue;
            tmpVal := FVertMaxVal - ((FVertMaxVal - FVertMinVal) / FVertDiv) * i;
            s := r2s(FVertDecNum, tmpVal) + FVertUnit;
            if i = 0 then
              VPos := FTopMargin
            else if i = FVertDiv then
              VPos := FTopMargin + GraphH - textHeight(s) + 2
            else
              VPos := FTopMargin + trunc((i * GraphH) / FVertDiv) - (textHeight(s) div 2);
            if (stLeft in FScaleText) then
              textOut(FLeftMargin - 4 - textWidth(s), VPos, s);
            if (stRight in FScaleText) then
              textOut(FLeftMargin + graphW + 4, VPos, s);
          end;
        end else begin
          s := FVertMaxText;
          VPos := FTopMargin;
          if (stLeft in FScaleText) then
            textOut(FLeftMargin - 4 - textWidth(s), VPos, s);
          if (stRight in FScaleText) then
            textOut(FLeftMargin + graphW + 4, VPos, s);
          s := FVertMinText;
          VPos := FTopMargin + GraphH - textHeight(s) + 2;
          if (stLeft in FScaleText) then
            textOut(FLeftMargin - 4 - textWidth(s), VPos, s);
          if (stRight in FScaleText) then
            textOut(FLeftMargin + graphW + 4, VPos, s);
        end;
      end;
      if (stTop in FScaleText) or (stBottom in FScaleText) then begin
        font.Size := FTextSize;
        font.Name := 'Arial';
        if colored then begin
          brush.Color := FBGColor;
          font.Color := FTextColor;
        end else begin
          brush.Color := clWhite;
          font.Color := clBlack;
        end;
        if (stUseHMinMax in FScaleText) then begin
          for i := 0 to FHorzDiv do begin
            if (i <> 0) and (i <> FHorzDiv) and (stHorzOnlyExtremity in FScaleText) then
              continue;
            if (i = 0) or (i = FHorzDiv) then begin
              tmpVal := FHorzMinVal + ((FHorzMaxVal - FHorzMinVal) / FHorzDiv) * i;
              s := r2s(FHorzDecNum, tmpVal);
            end else begin
              tmpVal := FHorzMinVal + ((FHorzMaxVal - FHorzMinVal) / FHorzDiv) * i;
              s := r2s(FHorzDecNum, tmpVal) + FHorzUnit;
            end;
            if i = 0 then
              HPos := FLeftMargin - 2
            else if i = FHorzDiv then
              HPos := FLeftMargin + GraphW - textWidth(s)
            else
              HPos := FLeftMargin + trunc((i * GraphW) / FHorzDiv) - (textWidth(s) div 2);
            if (stTop in FScaleText) then
              textOut(HPos, FTopMargin - 4 - textHeight(s), s);
            if (stBottom in FScaleText) then
              textOut(HPos, FTopMargin + graphH + 4, s);
          end;
        end else begin
          s := FHorzMinText;
          HPos := FLeftMargin - 2;
          if (stTop in FScaleText) then
            textOut(HPos, FTopMargin - 4 - textHeight(s), s);
          if (stBottom in FScaleText) then
            textOut(HPos, FTopMargin + graphH + 4, s);
          s := FHorzMaxText;
          HPos := FLeftMargin + GraphW - textWidth(s);
          if (stTop in FScaleText) then
            textOut(HPos, FTopMargin - 4 - textHeight(s), s);
          if (stBottom in FScaleText) then
            textOut(HPos, FTopMargin + graphH + 4, s);
        end;
      end;
    end;
  end;

  procedure DrawConceptionTraces;
  var
    s: string;
    i: integer;
    tmpColor: TColor;
    GraphH, GraphW: integer;
  begin
    graphW := GraphWidth;
    graphH := GraphHeight;
    With FBmpBuffer.Canvas do begin
      // traces, only in conception 
      if csDesigning in ComponentState then begin
        brush.Color := clLime;
        pen.Color := clLime;
        rectangle(FLeftMargin, FTopMargin + (GraphHeight div 10 * 2), FLeftMargin + (GraphWidth div 2), FTopMargin + GraphHeight);
        pen.Color := clRed;
        moveTo(GraphWidth div 10 * 7, FTopMargin + (GraphHeight div 10 * 4));
        lineTo(GraphWidth div 10 * 7, FTopMargin + GraphHeight);
        pen.Color := clYellow;
        moveTo(GraphWidth div 10 * 8, FTopMargin + (GraphHeight div 10 * 6));
        lineTo(GraphWidth div 10 * 8, FTopMargin + GraphHeight);
        brush.Color := clAqua;
        pen.Color := clAqua;
        rectangle(FLeftMargin + (GraphWidth div 10 * 9), FTopMargin + (GraphHeight div 10 * 3), FLeftMargin + (GraphWidth div 20 * 19), FTopMargin + GraphHeight);
        brush.Color := clBlack;
        font.size := FTextSize;
        font.Color := clAqua;
        s := 'Conception mode';
        textOut(FLeftMargin + 5, FTopMargin + (GraphHeight div 20 * 1), s);
      end;
    end;
  end;

  procedure DrawExecutionTraces;

    // take user value as argument and transform it as screen Position
    function GetHorzPos(r: real): integer;
    var
      HPos: integer;
    begin
      HPos := FLeftMargin + trunc((GraphWidth * r) / (FHorzMaxVal - FHorzMinVal));
      if HPos < FLeftMargin then
        HPos := FLeftMargin;
      if HPos > FLeftMargin + GraphWidth then
        HPos := FLeftMargin + GraphWidth;
      result := HPos;
    end;

    // take user value as argument and transform it as screen Position
    function GetVertPos(r: real): integer;
    var
      VPos: integer;
    begin
      VPos := FTopMargin + trunc(((FVertMaxVal - r) * GraphHeight) / (FVertMaxVal - FVertMinVal));
      if VPos > (FTopMargin + graphHeight - 1) then
        VPos := FTopMargin + graphHeight - 1;
      if VPos < FTopMargin then
        VPos := FTopMargin;
      result := VPos;
    end;

  var
    sArray: array[0..19] of string;
    s, sCmd, sTmp: string;
    i, iTmp: integer;
    rTmp: real;
    clTmp: TColor;
    X1, Y1, X2, Y2, X3, Y3, X4, Y4, X5, Y5: integer;
    graphW, graphH: integer;
  begin
    if LstValues.Count = 0 then exit;
    graphW := GraphWidth;
    graphH := GraphHeight;

    With FBmpBuffer.Canvas do begin

      // for each line of TStrings values, execute command
      for i := 0 to pred(LstValues.count) do begin

        // decode command and values in StringList LstValues...
        DecodeLine(LstValues[i], sArray);

        // cmd string is always at index 0 of the array
        sCmd := sArray[0];

        if sCmd = 'PEN.COLOR' then begin
          try
            sTmp := sArray[1];
            if sTmp = 'GRAPH_COLOR' then
              clTmp := GraphColor
            else if sTmp = 'BG_COLOR' then
              clTmp := BGColor
            else
              clTmp := StringToColor(sTmp);
            pen.Color := clTmp;
          except
          end;
        end else if sCmd = 'PEN.WIDTH' then begin
          try
            iTmp := strToInt(sArray[1]);
            pen.Width := iTmp;
          except
          end;
        end else if sCmd = 'BRUSH.COLOR' then begin
          try
            clTmp := StringToColor(sArray[1]);
            brush.Color := clTmp;
          except
          end;
        end else if sCmd = 'FONT.COLOR' then begin
          try
            clTmp := StringToColor(sArray[1]);
            font.Color := clTmp;
          except
          end;
        end else if sCmd = 'FONT.SIZE' then begin
          try
            iTmp := strToInt(sArray[1]);
            font.Size := iTmp;
          except
          end;
        end else if sCmd = 'FONT.NAME' then begin
          try
            sTmp := sArray[1];
            font.Name := sTmp;
          except
          end;
        end else if sCmd = 'LINETO' then begin
          try
            X1 := GetHorzPos(s2r(sArray[1]));
            Y1 := GetVertPos(s2r(sArray[2]));
            LineTo(X1, Y1);
          except
          end;
        end else if sCmd = 'MOVETO' then begin
          try
            X1 := GetHorzPos(s2r(sArray[1]));
            Y1 := GetVertPos(s2r(sArray[2]));
            MoveTo(X1, Y1);
          except
          end;
        end else if sCmd = 'RECTANGLE' then begin
          try
            X1 := GetHorzPos(s2r(sArray[1]));
            Y1 := GetVertPos(s2r(sArray[2]));
            X2 := GetHorzPos(s2r(sArray[3]));
            Y2 := GetVertPos(s2r(sArray[4]));
            Rectangle(X1, Y1, X2, Y2);
          except
          end;
        end else if sCmd = 'POLYGON4' then begin
          try
            X1 := GetHorzPos(s2r(sArray[1]));
            Y1 := GetVertPos(s2r(sArray[2]));
            X2 := GetHorzPos(s2r(sArray[3]));
            Y2 := GetVertPos(s2r(sArray[4]));
            X3 := GetHorzPos(s2r(sArray[5]));
            Y3 := GetVertPos(s2r(sArray[6]));
            X4 := GetHorzPos(s2r(sArray[7]));
            Y4 := GetVertPos(s2r(sArray[8]));
            Polygon([Point(X1, Y1), Point(X2, Y2), Point(X3, Y3), Point(X4, Y4)]);
          except
          end;
        end else if sCmd = 'POLYGON5' then begin
          try
            X1 := GetHorzPos(s2r(sArray[1]));
            Y1 := GetVertPos(s2r(sArray[2]));
            X2 := GetHorzPos(s2r(sArray[3]));
            Y2 := GetVertPos(s2r(sArray[4]));
            X3 := GetHorzPos(s2r(sArray[5]));
            Y3 := GetVertPos(s2r(sArray[6]));
            X4 := GetHorzPos(s2r(sArray[7]));
            Y4 := GetVertPos(s2r(sArray[8]));
            X5 := GetHorzPos(s2r(sArray[9]));
            Y5 := GetVertPos(s2r(sArray[10]));
            Polygon([Point(X1, Y1), Point(X2, Y2), Point(X3, Y3), Point(X4, Y4), Point(X5, Y5)]);
          except
          end;
        end else if sCmd = 'TEXTOUT' then begin
          try
            X1 := GetHorzPos(s2r(sArray[1]));
            Y1 := GetVertPos(s2r(sArray[2]));
            sTmp := sArray[3];
            Textout(X1, Y1, sTmp);
          except
          end;
        end else begin
          continue;
        end;

      end;
    end;
  end;

begin
  DrawBackGround;
  DrawGraphBackGround;
  DrawGrid;
  DrawScaleText;
  DrawConceptionTraces;
  DrawExecutionTraces;
end;

procedure TrmZGraph.Paint;
begin
  try
    UpdateBmpBuffer(true);
  except
  end;
  try
    Canvas.Draw(0, 0, FBmpBuffer);
  except
  end;
end;

procedure TrmZGraph.LoadDataFromFile(sFile: TFileName);
begin
  try
    LstValues.LoadFromFile(sFile);
    Paint;
  except
    showMessage('Can''t load data from file !');;
    Paint;
  end;
end;

procedure TrmZGraph.SaveDataToFile(sFile: TFileName);
begin
  if LstValues.count > 0 then begin
    try
      LstValues.SaveToFile(sFile);
      Paint;
    except
      showMessage('Can''t save data to file !');;
      Paint;
    end;
  end else
    showMessage('No data to save !');;
end;

procedure TrmZGraph.SaveGraphToFile(colored: boolean; sFile: TFileName);
begin
  try
    UpdateBmpBuffer(colored);
    if FBmpBuffer <> nil then begin
      try
        FBmpBuffer.SaveToFile(sFile);
      except
      end;
    end;
  except
    showMessage('Can''t save graph to file !');
  end;
  paint;
end;

procedure TrmZGraph.CopyGraphToClipboard(colored: boolean);
begin
  try
    UpdateBmpBuffer(colored);
    if FBmpBuffer <> nil then begin
      try
        Clipboard.Assign(FBmpBuffer);
      except
      end;
    end;
  except
    showMessage('Can''t copy graph to clipboard !');
  end;
  paint;
end;

procedure TrmZGraph.SetLstValues(value: TStringList);
begin
  if value <> LstValues then
  begin
    LstValues := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetBGColor(value: TColor);
begin
  if value <> FBGColor then
  begin
    FBGColor := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetBorder(value: TBorderType);
begin
  if value <> FBorder then
  begin
    FBorder := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetCTL3D(value: boolean);
begin
  if value <> FCTL3D then
  begin
    FCTL3D := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetGraphColor(value: TColor);
begin
  if value <> FGraphColor then
  begin
    FGraphColor := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetGridStyle(value: TPenStyle);
begin
  if value <> FGridStyle then
  begin
    FGridStyle := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetGridColor(value: TColor);
begin
  if value <> FGridColor then
  begin
    FGridColor := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetGridType(value: TGridType);
begin
  if value <> FGridType then
  begin
    FGridType := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetGridSubDiv(value: TGridSubDivType);
begin
  if value <> FGridSubDiv then
  begin
    FGridSubDiv := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetGridSubDivLen(value: word);
begin
  if value <> FGridSubDivLen then
  begin
    FGridSubDivLen := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetLeftMargin(value: integer);
begin
  if value <> FLeftMargin then
  begin
    FLeftMargin := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetRightMargin(value: integer);
begin
  if value <> FRightMargin then
  begin
    FRightMargin := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetTopMargin(value: integer);
begin
  if value <> FTopMargin then
  begin
    FTopMargin := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetBottomMargin(value: integer);
begin
  if value <> FBottomMargin then
  begin
    FBottomMargin := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetHorzMinVal(value: real);
begin
  if value <> FHorzMinVal then
  begin
    FHorzMinVal := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetHorzMaxVal(value: real);
begin
  if value <> FHorzMaxVal then
  begin
    FHorzMaxVal := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetHorzMinText(value: string);
begin
  if value <> FHorzMinText then
  begin
    FHorzMinText := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetHorzMaxText(value: string);
begin
  if value <> FHorzMaxText then
  begin
    FHorzMaxText := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetHorzUnit(value: string);
begin
  if value <> FHorzUnit then
  begin
    FHorzUnit := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetHorzDecNum(value: integer);
begin
  if value <> FHorzDecNum then
  begin
    FHorzDecNum := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetHorzDiv(value: integer);
begin
  if value <> FHorzDiv then
  begin
    FHorzDiv := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetVertMinVal(value: real);
begin
  if value <> FVertMinVal then
  begin
    FVertMinVal := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetVertMaxVal(value: real);
begin
  if value <> FVertMaxVal then
  begin
    FVertMaxVal := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetVertMinText(value: string);
begin
  if value <> FVertMinText then
  begin
    FVertMinText := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetVertMaxText(value: string);
begin
  if value <> FVertMaxText then
  begin
    FVertMaxText := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetVertUnit(value: string);
begin
  if value <> FVertUnit then
  begin
    FVertUnit := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetVertDecNum(value: integer);
begin
  if value <> FVertDecNum then
  begin
    FVertDecNum := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetVertDiv(value: integer);
begin
  if value <> FVertDiv then
  begin
    FVertDiv := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetScaleText(value: TScaleText);
begin
  if value <> FScaleText then
  begin
    FScaleText := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetTextColor(value: TColor);
begin
  if value <> FTextColor then
  begin
    FTextColor := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetTextSize(value: integer);
begin
  if value <> FTextSize then
  begin
    FTextSize := value;
    Paint;
  end;
end;

procedure TrmZGraph.SetMaxItems(value: integer);
var
  i: integer;
begin
  if value <> FMaxItems then
  begin
    FMaxItems := value;
    if (LstValues.Count > FMaxItems) and (FMaxItems > 0) then
      for i := 1 To (LstValues.Count - FMaxItems) do
        LstValues.Delete(0);
    Paint;
  end;
end;

procedure TrmZGraph.Clear;
begin
  LstValues.clear;
  Paint;
end;

function TrmZGraph.AddValues(s: string; refresh: boolean): integer;
var
  i: integer;
begin
  if (LstValues.Count > FMaxItems) and (FMaxItems > 0) then exit;
  LstValues.add(s);
  result := LstValues.count;
  if refresh then Paint;
end;

function TrmZGraph.DeleteValues(i: integer; refresh: boolean): integer;
begin
  if i > LstValues.Count - 1 then exit;
  LstValues.delete(i);
  result := LstValues.count;
  if refresh then Paint;
end;

function TrmZGraph.GetValues(x, y: integer; var ValArray: array of real): byte;
var
  s: string;
  i, xPos, sep: integer;
  val: real;
begin
  for i := 0 to 7 do ValArray[i] := 0;
  xPos := x - FLeftMargin;
  if (xPos > lstValues.Count - 1) or (xPos < 0) then begin
    // showmessage('Out of range !');
    exit;
  end;
  s := LstValues[xPos];
  // if FValuePosition = vpPixels then begin
    for i := 0 to 7 do begin
      sep := pos(';', s);
      if sep > 0 then begin
        val := s2r(copy(s, 1, sep - 1));
        delete(s, 1, sep);
      end else if (sep = 0) and (s <> '') then begin
        val := s2r(s);
        s := '';
      end;
      ValArray[i] := val;
      if s = '' then break;
    end;
  // end;
end;

function TrmZGraph.GetHorzValueFromXPos(x: integer): real;
var
  ratio: real;
begin
  ratio := (HorzMaxVal - HorzMinVal) / graphWidth;
  result := ((x - LeftMargin + 1) * ratio) + HorzMinVal;
end;

function TrmZGraph.GetVertValueFromYPos(y: integer): real;
var
  ratio: real;
begin
  ratio := (VertMaxVal - VertMinVal) / graphHeight;
  result := ((Height - BottomMargin - y) * ratio) + VertMinVal;
end;

function TrmZGraph.GraphBottom: integer;
begin
  result := Height - FBottomMargin;
end;

function TrmZGraph.GraphLeft: integer;
begin
  result := FLeftMargin;
end;

function TrmZGraph.GraphRight: integer;
begin
  result := width - FRightMargin;
end;

function TrmZGraph.GraphTop: integer;
begin
  result := FTopMargin;
end;

function TrmZGraph.GraphHeight: integer;
begin
  result := height - FBottomMargin - FTopMargin;
end;

function TrmZGraph.GraphWidth: integer;
begin
  result := width - FLeftMargin - FRightMargin;
end;

end.

